@@ -8,6 +8,8 @@ use Compiler::Parser::Node::Block;
 use Compiler::Parser::Node::Module;
 use Compiler::Parser::Node::Package;
 use Compiler::Parser::Node::RegPrefix;
+use Compiler::Parser::Node::RegReplace;
+use Compiler::Parser::Node::Regexp;
 use Compiler::Parser::Node::ForStmt;
 use Compiler::Parser::Node::ForeachStmt;
 use Compiler::Parser::Node::WhileStmt;
@@ -31,7 +33,7 @@ our @ISA = qw(Exporter);
 our %EXPORT_TAGS = ( 'all' => [ qw() ] );
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT = qw();
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 require XSLoader;
 XSLoader::load('Compiler::Parser', $VERSION);
 
@@ -115,7 +117,7 @@ Compiler::Parser creates abstract syntax tree for perl5.
 =item my $ast = $parser->parse($tokens);
 
     Get array reference includes abstract syntax tree each statement.
-    This method requires `$tokens` from Compiler::Lexer::tokenize.
+    This method requires $tokens from Compiler::Lexer::tokenize.
 
 =item my $renderer = Compiler::Parser::AST::Renderer->new();
 
@@ -124,7 +126,7 @@ Compiler::Parser creates abstract syntax tree for perl5.
 =item $renderer->render($ast)
 
     Render abstract syntax tree.
-    This method requires `$ast` from Compiler::Parser::parse.
+    This method requires $ast from Compiler::Parser::parse.
     Default rendering engine is Compiler::Parser::AST::Renderer::Engine::Text.
 
 =back
